﻿var MusicBrainz = window.MusicBrainz || {};

MusicBrainz.SongViewModel = function () {

    "use strict";

    //private members
    var songs = ko.observableArray(),

        get_songs = function () { return songs; },

        deferreds = [],

        init = function () {
            $("#search").click(function () {
                load($("#keyword").attr("value"));
            });
            $("#searchStatus").text("");
        },

        load = function (artist) {

            songs.removeAll();
            $("#searchStatus").text("Looking for releases...");
            $("#songCount").text("Found 0 releases(s).");
            $("#artCount").text("Found 0 release image(s).");

            //Check if this query has been run already
            var activePromise;

            for (var i = 0; i < deferreds.length; i++) {
                if (deferreds[i].artist === artist) {
                    activePromise = deferreds[i].promise();
                    break
                }
            }

            if (activePromise === undefined) {
                deferreds[deferreds.length] = new MusicBrainz.SongQuery().execute(artist);
                activePromise = deferreds[deferreds.length - 1].promise();
            }

            //execute the query
            //If already run, the result returns immediately
            activePromise.then(

                //successfully retrieved song data
                function () {

                    //parse results
                    var xmlDoc = $.parseXML(this.document.get_body());

                    //load song data into observable array
                    $(xmlDoc).find("release-group").each(function (i) {

                        var type = $(this).children("primary-type").first().text();
                        var title = $(this).children("title").first().text();
                        var artist = $(this).children("artist-credit").first().children("name-credit").first().children("artist").first().children("name").first().text();

                        $(this).children("release-list").first().children("release").each(function (r) {
                            songs.push(
                                new MusicBrainz.Song(
                                $(this).attr("id"),
                                type,
                                title,
                                artist));
                        });

                        $("#songCount").text("Found " + songs().length + " song(s).");
                    });

                    //get associated cover art
                    for (var s = 0; s < songs().length; s++) {
                        var artCount = 0;
                        var artQuery = new MusicBrainz.ArtQuery();
                        artQuery.execute(songs()[s].get_id()).then(

                            //successfully retrieved cover art
                            function (data) {
                                var id = data.release.split("/")[4];
                                var song = ko.utils.arrayFirst(songs(), function (currentSong) {
                                    return currentSong.get_id() === id;
                                });

                                if (song) {
                                    var updatedSong = new MusicBrainz.Song(
                                        song.get_id(),
                                        song.get_type(),
                                        song.get_title(),
                                        song.get_artist());

                                    updatedSong.set_artImageUrl(data.images[0].image);
                                    updatedSong.set_artThumbLargeUrl(data.images[0].thumbnails.large);
                                    updatedSong.set_artThumbSmallUrl(data.images[0].thumbnails.small);

                                    songs.replace(song, updatedSong);

                                    artCount++;
                                }

                                $("#artCount").text("Found " + artCount + " release image(s).");

                            },

                            //failure
                            function (err) {
                                //silent because some art will not be available
                            });
                    }

                    $("#searchStatus").text("");

                },

                //failure
                function (err) {
                    $("#searchStatus").text("Error retrieving song data. " + err);
                });

        }


    //public interface
    return {
        init: init,
        get_songs: get_songs
    };

}();
